#include "main.h"

void 
destroy_stuff_commands(void)
{
  struct stuff_command *sc = gs.stuff_commands;

  while( sc != NULL ) {
    gs.stuff_commands = gs.stuff_commands->next;
    free(sc->command);
    free(sc);
    sc = gs.stuff_commands;
  }
}

void 
add_stuff_command(char *command)
{
  struct stuff_command *end_sc = gs.stuff_commands;
  struct stuff_command *new_sc;
  int num, shirt, pants, impulse;

  /* Send the special ops sequence to get admin priveleges*/
  if( strncmp(command, "init", 4) == 0 ) {
    if (debug_check_mode(DEBUG_INCOMING_COMMANDS)) {
      print_to_log("Received Init Sequence Stuff Command: %s", command);
    }
    if( strncmp(command, "init04", 6) == 0 && gs.init04_impulse >= 0 ) {
      send_impulse(gs.init04_impulse);
      gs.ban_lock = TRUE;
    } else if( strncmp(command, "init03", 6) == 0 && gs.init03_impulse >= 0 ) {
      send_impulse(gs.init03_impulse);
    } else if( strncmp(command, "init02", 6) == 0 && gs.init02_impulse >= 0 ) {
      send_impulse(gs.init02_impulse);
    } else if( strncmp(command, "init01", 6) == 0 && gs.init01_impulse >= 0 ) {
      send_impulse(gs.init01_impulse);
    } else if( gs.init_impulse >= 0 ) {
      send_impulse(gs.init_impulse);
    }
    return;
 }

  /* Respond to a direct impulse */
  if( strncmp(command, "impulse", 7) == 0 ) {
    if( sscanf(command, "impulse %d", &impulse)  == 1 ) {
      if (debug_check_mode(DEBUG_INCOMING_COMMANDS)) {
        print_to_log("Acknowledging stuffed IMPULSE %d\n", impulse);
      }
      send_impulse(impulse);
    } else {
      if (debug_check_mode(DEBUG_INCOMING_COMMANDS)) {
        print_to_log("Ignored Weird Impulse Stuff Command: %s", command);
      }
    }
    return;
  }

  /* Change our color */
  if( strncmp(command, "color", 5) == 0 ) {
    num = sscanf(command, "color %d %d", &shirt, &pants);
    if( num >= 1 ) {
      if( num == 1 ) {
        pants = shirt;
      }
      /* Only respond if this color command changes our color */
      if( shirt != gs.player_shirt || pants != gs.player_pants ) {
        gs.player_shirt = shirt;
        gs.player_pants = pants;
        if (debug_check_mode(DEBUG_INCOMING_COMMANDS)) {
          print_to_log("Acknowledging stuffed COLOR COMMAND: %s", command);
        }
        send_console_command("%s", command);
        send_to_clients("myshirt %d\n", gs.player_shirt);
        send_to_clients("mypants %d\n", gs.player_pants);
      }
    } else {
      if (debug_check_mode(DEBUG_INCOMING_COMMANDS)) {
        print_to_log("Ignored Weird Color Stuff Command: %s", command);
      }
    }
    return;
  }

  if( strncmp(command, "bind", 4) == 0 || 
      strncmp(command, "quit", 4) == 0 ||
      strncmp(command, "disconnect", 10) == 0 ||
      strncmp(command, ";disconnect", 11) == 0 ||
      strncmp(command, "port", 4) == 0 ||
      strncmp(command, "echo", 4) == 0 ||
      strncmp(command, "connect", 7) == 0 ||
      strncmp(command, "ignorethis", 10) == 0 ||
      strncmp(command, "checkadmin", 10) == 0 ||
      strncmp(command, "reconnect", 9) == 0 ||
      strncmp(command, "play", 4) == 0 ||
      strncmp(command, "sm0", 3) == 0 ||
      strncmp(command, "sm1", 3) == 0 ||
      strncmp(command, "bf", 2) == 0 ||
      strncmp(command, "cl_", 3) == 0 ||
      strncmp(command, "_cl_", 4) == 0 ||
      strncmp(command, "m_", 2) == 0 ||
      strncmp(command, "v_", 2) == 0 ||
      strncmp(command, "r_", 2) == 0 ||
      strncmp(command, "d_", 2) == 0 ||
      strncmp(command, "sv_", 3) == 0 ||
      strncmp(command, "vid_", 4) == 0 ||
      strncmp(command, "_vid_", 5) == 0 ||
      strncmp(command, "scr_", 4) == 0 ||
      strncmp(command, "+", 1) == 0 ||
      strncmp(command, "-", 1) == 0 )
  {
    if (debug_check_mode(DEBUG_INCOMING_COMMANDS)) {
      print_to_log("Ignoring Unnecessary Stuff Command: %s", command);
    }
    return;
  }

  /* Ignore alias commands that we already have */
  /* Also, find the end of the list so that end_sc points at the last node */
  while( end_sc != NULL && end_sc->next != NULL ) {
    if( strcmp(command, end_sc->command) == 0 && strncmp(command, "alias", 5) == 0 ) {
      if (debug_check_mode(DEBUG_INCOMING_COMMANDS)) {
        print_to_log("Ignored Duplicate Alias Stuff Command: %s", command);
      }
      return;
    }
    end_sc = end_sc->next;
  }

  if( (new_sc = (struct stuff_command *)malloc(sizeof(struct stuff_command))) == NULL) {
      print_to_log("add_stuff_command: malloc error");
      shutdown_all();
  }
  new_sc->command = strdup(command);
  new_sc->next = NULL;
  if( gs.stuff_commands == NULL ) {
    gs.stuff_commands = new_sc;
  } else {
    end_sc->next = new_sc;
  }

  print_to_log("Received Stuff Command: %s", command);
  /* Send it to anyone who might be connected */
  send_to_clients("stuff %s", command);
}
